let version = "%%VERSION%%"

open Logarion
module A = Archive.Make (Logarion.File_store)

let file_when_changed source dest fn title text =
  if (try Unix.((stat dest).st_mtime < (stat source).st_mtime) with _-> true) then
    File_store.file dest (fn title text)

let writer dir name acc (text,store_item) = (* todo: single_parser -> [files] *)
  let open Logarion in
  match Text.(str "Content-Type" text) with
  | "" | "text/plain" ->
    let filename = dir ^ "/" ^ Text.alias text in
(*    let idfilename = dir ^ "/" ^ Id.to_string text.Text.uuid ^ ".txt" in*)
(*    file_when_changed store_item idfilename (fun _title -> Text.to_string) text.title text;*)
    file_when_changed store_item (filename ^ ".html") (Html.page "") name text;
    file_when_changed store_item (filename ^ ".gmi") Gemini.page name text;
    (acc + 1)
  | x -> prerr_endline ("No converter for Content-Type: " ^ x ^ ", for " ^ text.Text.title); acc

let index_writer dir archive topic_roots topic_map indexed_texts =
  let name = archive.A.name in
  let file path = File_store.file (dir ^ path) in
  file "/index.pck"  (Header_pack.pack archive indexed_texts);
  file "/index.html" (Html.topic_main_index name topic_roots indexed_texts);
  List.iter
    (fun topic -> file ("/index." ^ topic ^ ".htm") (Html.topic_sub_index name topic_map topic indexed_texts))
    topic_roots;
  file "/index.date.html" (Html.date_index name indexed_texts);
  let base_url = try Store.KV.find "HTTP-URL" archive.File_store.kv
    with Not_found -> prerr_endline "Missing `HTTP-URL:` in config"; ""
  in
  file "/feed.atom" (Atom.feed archive.A.name archive.A.id base_url indexed_texts);
  file "/index.date.gmi" (Gemini.date_index name indexed_texts)

let convert_all dir archive =
  let name = archive.A.name in
  let count = A.(fold ~order:newest (writer dir name) 0 archive) in (*TODO: merge*)
  let topic_map, indexed_metas =
    let topic_map = Logarion.Topic_set.Map.empty in
    let fn (ts,ls) (elt,_) =
    	Logarion.(Topic_set.to_map ts (Text.set "topics" elt)), elt::ls in
    A.(fold ~order:newest fn (topic_map,[]) archive) in
  let topic_roots = Logarion.Topic_set.roots topic_map in
  index_writer dir archive topic_roots topic_map indexed_metas;
  print_endline @@ "Converted: " ^ string_of_int (count) ^ "\nIndexed: " ^ string_of_int (List.length indexed_metas);
  Ok ()

let convert_dir cmd_dir =
  let (>>=) = Result.bind in
  let config_dir archive = try Ok (Store.KV.find "Export-Dir" archive.File_store.kv) with Not_found -> Error "unspecified export dir" in
  let init dir =
    Result.map_error (function `Msg m -> m)
    Logarion.File_store.Directory.(directory dir |> print ~descr:"export" dir) in
  (A.of_path "."
    >>= fun archive -> (match cmd_dir with "" -> config_dir archive | x -> Ok x)
    >>= fun dir -> init dir
    >>= fun _ -> convert_all dir archive)
   |> function Ok () -> () | Error x -> prerr_endline x

open Cmdliner

let term =
  let directory = Arg.(value & pos 0 string "" & info [] ~docv:"target directory" ~doc:"Directory to convert into") in
  Term.(const convert_dir $ directory),
  Term.info
    "convert" ~doc:"convert archive"
    ~man:[ `S "DESCRIPTION"; `P "Convert Logarion archive" ]
